(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[357],{1211:function(t,r,n){Promise.resolve().then(n.bind(n,47424))},97500:function(t,r,n){"use strict";async function e(t,r){let{arg:n}=r,e=await fetch(""+t,{method:"POST",headers:{"Content-Type":"application/json"},body:JSON.stringify(n)});if(!(e.status>=200&&e.status<300))throw Error(await e.text());let o=await e.json();if(!e.ok)throw Error(o.message);return o}n.d(r,{_i:function(){return o},gz:function(){return s},hw:function(){return i},sj:function(){return a},wG:function(){return e}});let o=async function(){for(var t=arguments.length,r=Array(t),n=0;n<t;n++)r[n]=arguments[n];let e=await fetch(""+r[0],r[1]);if(!e.ok)throw Error(await e.text());return e.json()},a=async(t,r)=>{let n=await fetch(""+t,r);if(!n.ok)throw Error(await n.text());return n};async function i(t,r){let{arg:n}=r,e=await fetch("".concat("").concat(t,"/").concat(n),{method:"DELETE"});if(!e.ok)throw Error(await e.text());return e}async function s(t,r){let{arg:n}=r,e=await fetch("".concat("").concat(t),{method:"PUT",headers:{"Content-Type":"application/json"},body:JSON.stringify(n)});if(!e.ok)throw Error(await e.text());return e}},47424:function(t,r,n){"use strict";n.r(r),n.d(r,{default:function(){return u}});var e=n(57437),o=n(76301),a=n(85323),i=n(97500),s=n(42337),c=n(72046);function u(){let{Header:t,Footer:r,Sider:n,Content:u}=o.Layout,{data:l,error:d,isLoading:f}=(0,a.ZP)("/v1/status",i._i);return f?(0,e.jsx)(o.yC,{size:"large"}):(0,e.jsxs)(e.Fragment,{children:[(0,e.jsx)(t,{style:{backgroundColor:"var(--semi-color-bg-1)"},children:(0,e.jsx)(o.JL,{style:{border:"none"},header:(0,e.jsxs)(e.Fragment,{children:[(0,e.jsx)("div",{style:{backgroundColor:"rgba(var(--semi-lime-2), 1)",borderRadius:"var(--semi-border-radius-large)",color:"var(--semi-color-bg-0)",display:"flex",padding:"6px"},children:(0,e.jsx)(s.Z,{size:"large"})}),(0,e.jsx)("h4",{style:{marginLeft:"12px"},children:"任务平台"})]}),mode:"horizontal"})}),(0,e.jsx)(u,{style:{paddingLeft:12,paddingRight:12,backgroundColor:"var(--semi-color-bg-0)"},children:(0,e.jsx)("main",{children:(0,e.jsx)(c.L,{data:l})})})]})}}},function(t){t.O(0,[308,385,251,283,191,147,429,301,323,80,971,117,744],function(){return t(t.s=1211)}),_N_E=t.O()}]);